@@ -1588,9 +1588,9 @@ item_update_children_uri (TrackerMinerFS    *fs,
 
 	sparql = g_strdup_printf ("SELECT ?child ?url nie:mimeType(?child) WHERE { "
 				  "  ?child nie:url ?url . "
-                                  "  FILTER (tracker:uri-is-descendant (\"%s\", ?uri)) "
+                                  "  FILTER (tracker:uri-is-descendant (\"%s\", ?url)) "
 				  "}",
-				  uri);
+				  source_uri);
 
 	tracker_miner_execute_sparql (TRACKER_MINER (fs),
 	                              sparql,
@@ -2155,7 +2155,7 @@ ensure_mtime_cache (TrackerMinerFS *fs,
 	CacheQueryData data;
 	GFile *parent;
 
-        if (G_UNLIKELY (!fs->private->mtime_cache)) {
+	if (G_UNLIKELY (!fs->private->mtime_cache)) {
 		fs->private->mtime_cache = g_hash_table_new_full (g_file_hash,
 		                                                  (GEqualFunc) g_file_equal,
 		                                                  (GDestroyNotify) g_object_unref,
@@ -2182,12 +2182,11 @@ ensure_mtime_cache (TrackerMinerFS *fs,
 
 	g_debug ("Generating mtime cache for folder: %s", uri);
 
-	query = g_strdup_printf ("SELECT ?uri ?time { "
-	                         "  ?u nfo:fileLastModified ?time ; "
-	                         "     nie:url ?uri . "
-	                         "  FILTER (tracker:uri-is-parent (\"%s\", ?uri)) "
-	                         "}",
-	                         uri);
+	query = g_strdup_printf ("SELECT ?url ?last { ?u nfo:belongsToContainer ?p ; "
+	                                                "nie:url ?url ; "
+	                                                "nfo:fileLastModified ?last . "
+	                                             "?p nie:url \"%s\" }", uri);
+
 	g_free (uri);
 
 	data.main_loop = g_main_loop_new (NULL, FALSE);
