@@ -46,6 +46,30 @@
 #define RDF_PREFIX TRACKER_RDF_PREFIX
 #define RDF_TYPE RDF_PREFIX "type"
 
+/* I *know* that this is some arbitrary number that doesn't seem to
+ * resemble anything. In fact it's what I experimentally measured to
+ * be a good value on a default Debian testing which has
+ * max_message_size set to 1 000 000 000 in session.conf. I didn't have
+ * the feeling that this value was very much respected, as the size
+ * of the DBusMessage when libdbus decided to exit() the process was
+ * around 160 MB, and not ~ 1000 MB. So if you take 160 MB and you
+ * devide it by 1000000 you have an average string size of ~ 160
+ * bytes plus DBusMessage's overhead. If that makes this number less
+ * arbitrary for you, then fine.
+ *
+ * I really hope that the libdbus people get to their senses and 
+ * either stop doing their exit() nonsense in a library, and instead
+ * return a clean DBusError or something, or create crystal clear
+ * clarity about the maximum size of a message. And make it both so
+ * that I can get this length at runtime (without having to parse
+ * libdbus's own configuration files) and my DBusMessage's current
+ * total length. As far as I know are both not possible. So that for
+ * me means that libdbus's exit() is unacceptable.
+ *
+ * Note for the debugger of the future, the "Disconnected" signal gets
+ * sent to us by the bus, which in turn makes libdbus-glib perform exit(). */
+
+#define DBUS_ARBITRARY_MAX_MSG_SIZE 1000000
 
 G_DEFINE_TYPE(TrackerResources, tracker_resources, G_TYPE_OBJECT)
 
@@ -63,8 +87,14 @@ typedef struct {
 typedef struct {
 	DBusGMethodInvocation *context;
 	guint request_id;
+	DBusMessage *reply;
 } TrackerDBusMethodInfo;
 
+typedef struct {
+	DBusMessage *reply;
+	GError *error;
+	gpointer user_data;
+} InThreadPtr;
 
 static void tracker_resources_finalize (GObject *object);
 
@@ -152,7 +182,7 @@ tracker_resources_load (TrackerResources         *object,
                         GError                  **error)
 {
 	TrackerDBusMethodInfo   *info;
-	guint               request_id;
+	guint                    request_id;
 	GFile  *file;
 
 	request_id = tracker_dbus_get_next_request_id ();
@@ -179,28 +209,111 @@ tracker_resources_load (TrackerResources         *object,
 }
 
 static void
-query_callback (TrackerDBResultSet *result_set, GError *error, gpointer user_data)
+query_callback (gpointer inthread_data, GError *error, gpointer user_data)
 {
+	InThreadPtr *ptr = inthread_data;
 	TrackerDBusMethodInfo *info = user_data;
-	GPtrArray *values;
 
-	if (error) {
+	if (ptr && ptr->error) {
+		tracker_dbus_request_failed (info->request_id,
+		                             info->context,
+		                             &ptr->error,
+		                             NULL);
+		dbus_g_method_return_error (info->context, ptr->error);
+		g_error_free (ptr->error);
+	} else if (error) {
 		tracker_dbus_request_failed (info->request_id,
 		                             info->context,
 		                             &error,
 		                             NULL);
 		dbus_g_method_return_error (info->context, error);
-		return;
+	} else {
+		tracker_dbus_request_success (info->request_id,
+		                              info->context);
+
+		dbus_g_method_send_reply (info->context, ptr->reply);
 	}
 
-	tracker_dbus_request_success (info->request_id,
-	                              info->context);
+	if (ptr)
+		g_slice_free (InThreadPtr, ptr);
+}
 
-	values = tracker_dbus_query_result_to_ptr_array (result_set);
+static gpointer
+query_inthread (TrackerDBCursor *cursor, GError *error, gpointer user_data)
+{
+	InThreadPtr *ptr = g_slice_new0 (InThreadPtr);
+	TrackerDBusMethodInfo *info = user_data;
+	DBusMessage *reply;
+	DBusMessageIter iter, rows_iter;
+	guint cols;
+	GError *loop_error = NULL;
+	guint length = 0;
+	gboolean cont;
 
-	dbus_g_method_return (info->context, values);
+	if (error) {
+		ptr->error = g_error_copy (error);
+		return ptr;
+	}
 
-	tracker_dbus_results_ptr_array_free (&values);
+	reply = info->reply;
+
+	dbus_message_iter_init_append (reply, &iter);
+
+	cols = tracker_db_cursor_get_n_columns (cursor);
+
+	dbus_message_iter_open_container (&iter,
+	                                  DBUS_TYPE_ARRAY,
+	                                  "as",
+	                                  &rows_iter);
+
+	cont = TRUE;
+
+	while (tracker_db_cursor_iter_next (cursor, &loop_error) && cont) {
+		DBusMessageIter cols_iter;
+		guint i;
+
+		if (loop_error != NULL) {
+			break;
+		}
+
+		dbus_message_iter_open_container (&rows_iter,
+		                                  DBUS_TYPE_ARRAY,
+		                                  "s",
+		                                  &cols_iter);
+
+		for (i = 0; i < cols && cont; i++, length++) {
+			const gchar *result_str;
+			result_str = tracker_db_cursor_get_string (cursor, i);
+
+			if (result_str == NULL)
+				result_str = "";
+
+			dbus_message_iter_append_basic (&cols_iter, DBUS_TYPE_STRING, &result_str);
+
+			if (length > DBUS_ARBITRARY_MAX_MSG_SIZE) {
+				g_set_error (&loop_error,
+				             TRACKER_DB_INTERFACE_ERROR,
+				             TRACKER_DB_INTERRUPTED,
+				             "result set of the query is too large");
+
+				cont = FALSE;
+			}
+
+		}
+
+		dbus_message_iter_close_container (&rows_iter, &cols_iter);
+	}
+
+	dbus_message_iter_close_container (&iter, &rows_iter);
+
+	if (loop_error) {
+		ptr->error = loop_error;
+		ptr->reply = NULL;
+	} else {
+		ptr->reply = reply;
+	}
+
+	return ptr;
 }
 
 void
@@ -210,12 +323,9 @@ tracker_resources_sparql_query (TrackerResources         *self,
                                 GError                  **error)
 {
 	TrackerDBusMethodInfo   *info;
-	TrackerResourcesPrivate *priv;
 	guint                 request_id;
 	gchar                 *sender;
 
-	priv = TRACKER_RESOURCES_GET_PRIVATE (self);
-
 	request_id = tracker_dbus_get_next_request_id ();
 
 	tracker_dbus_async_return_if_fail (query != NULL, context);
@@ -230,11 +340,12 @@ tracker_resources_sparql_query (TrackerResources         *self,
 
 	info->request_id = request_id;
 	info->context = context;
+	info->reply = dbus_g_method_get_reply (context);
 
 	sender = dbus_g_method_get_sender (context);
 
 	tracker_store_sparql_query (query, TRACKER_STORE_PRIORITY_HIGH,
-	                            query_callback, sender,
+	                            query_inthread, query_callback, sender,
 	                            info, destroy_method_info);
 
 	g_free (sender);
@@ -266,12 +377,9 @@ tracker_resources_sparql_update (TrackerResources        *self,
                                  GError                 **error)
 {
 	TrackerDBusMethodInfo   *info;
-	TrackerResourcesPrivate *priv;
 	guint                 request_id;
 	gchar                 *sender;
 
-	priv = TRACKER_RESOURCES_GET_PRIVATE (self);
-
 	request_id = tracker_dbus_get_next_request_id ();
 
 	tracker_dbus_async_return_if_fail (update != NULL, context);
@@ -322,12 +430,9 @@ tracker_resources_sparql_update_blank (TrackerResources       *self,
                                        GError                **error)
 {
 	TrackerDBusMethodInfo   *info;
-	TrackerResourcesPrivate *priv;
 	guint                 request_id;
 	gchar                 *sender;
 
-	priv = TRACKER_RESOURCES_GET_PRIVATE (self);
-
 	request_id = tracker_dbus_get_next_request_id ();
 
 	tracker_dbus_async_return_if_fail (update != NULL, context);
@@ -379,12 +484,9 @@ tracker_resources_batch_sparql_update (TrackerResources          *self,
                                        GError                   **error)
 {
 	TrackerDBusMethodInfo   *info;
-	TrackerResourcesPrivate *priv;
 	guint                 request_id;
 	gchar                 *sender;
 
-	priv = TRACKER_RESOURCES_GET_PRIVATE (self);
-
 	request_id = tracker_dbus_get_next_request_id ();
 
 	tracker_dbus_async_return_if_fail (update != NULL, context);
@@ -428,12 +530,9 @@ tracker_resources_batch_commit (TrackerResources         *self,
                                 GError                  **error)
 {
 	TrackerDBusMethodInfo *info;
-	TrackerResourcesPrivate *priv;
 	guint                 request_id;
 	gchar                 *sender;
 
-	priv = TRACKER_RESOURCES_GET_PRIVATE (self);
-
 	request_id = tracker_dbus_get_next_request_id ();
 
 	tracker_dbus_request_new (request_id,
