@@ -27,7 +27,7 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
-#include <libtracker-client/tracker-client.h>
+#include <libtracker-client/tracker.h>
 #include <libtracker-common/tracker-common.h>
 
 #define ABOUT	  \
@@ -162,6 +162,7 @@ get_fts_string (GStrv    search_words,
 
 	for (i = 0; i < len; i++) {
 		g_string_append (fts, search_words[i]);
+		g_string_append_c (fts, '*');
 
 		if (i < len - 1) {
 			if (use_or_operator) {
@@ -976,18 +977,18 @@ get_all_by_search_foreach (gpointer value,
 		if (mime_type && mime_type[0] == '\0') {
 			mime_type = NULL;
 		}
-
+		
 		if (mime_type) {
 			g_print ("  %s\n"
 			         "    %s\n"
-			         "    %s\n",
-			         urn,
-			         mime_type,
+			         "    %s\n", 
+			         urn, 
+			         mime_type, 
 			         class);
 		} else {
 			g_print ("  %s\n"
-			         "    %s\n",
-			         urn,
+			         "    %s\n", 
+			         urn, 
 			         class);
 		}
 	}
@@ -1153,48 +1154,42 @@ main (int argc, char **argv)
 		g_thread_init (NULL);
 	}
 
+	/* Check terms are not stopwords */
 	if (terms) {
 		TrackerLanguage *language;
-		gboolean stop_words_found;
+		GHashTable *stop_words;
+		const gchar *stop_word_found;
 		gchar **p;
 
-		/* Check terms don't have additional quotes */
-		for (p = terms; *p; p++) {
-			gchar *term = *p;
-			gint end = strlen (term) - 1;
-
-			if ((term[0] == '"' && term[end] == '"') ||
-			    (term[0] == '\'' && term[end] == '\'')) {
-				/* We never have a quote JUST at the end */
-				term[0] = term[end] = ' ';
-				g_strstrip (term);
-			}
-		}
-
-		/* Check if terms are stopwords, and warn if so */
 		language = tracker_language_new (NULL);
-		stop_words_found = FALSE;
-		for (p = terms; *p; p++) {
-			gchar *down;
+		stop_words = tracker_language_get_stop_words (language);
 
-			down = g_utf8_strdown (*p, -1);
+		for (p = terms, stop_word_found = NULL; *p && !stop_word_found; p++) {
+			gpointer data;
+			gchar *up = g_utf8_strdown (*p, -1);
 
-			if (tracker_language_is_stop_word (language, down)) {
-				g_printerr (_("Search term '%s' is a stop word."),
-				            down);
-				g_printerr ("\n");
-
-				stop_words_found = TRUE;
+			data = g_hash_table_lookup (stop_words, up);
+			if (data) {
+				stop_word_found = *p;
 			}
-
-			g_free (down);
+			g_free (up);
 		}
 
-		if (stop_words_found) {
-			g_printerr (_("Stop words are common words which "
-			              "may be ignored during the indexing "
-			              "process."));
+		if (stop_word_found) {
+			g_printerr (_("Search term '%s' is a stop word."),
+			            stop_word_found);
 			g_printerr ("\n\n");
+			g_printerr (_("Stop words are common words which are "
+			              "ignored during the indexing process."));
+			g_printerr ("\n");
+			g_printerr (_("This means this search term will never "
+			              "be found when matching FTS entries."));
+			g_printerr ("\n\n");
+
+			g_option_context_free (context);
+			g_object_unref (language);
+
+			return EXIT_FAILURE;
 		}
 
 		g_object_unref (language);
